module Planik
  module Arbeitszeitkonto

  ##
  # Definiert eine Zeitspanne einer bestimmten Zeitkontos (Sa, den 03.08.15 von 22 - 24) oder eines Dienstes.
  # Kann die Ueberlappung von zwei Zeitspannen berechnen
  ##
  class Zeitspanne
    attr_reader :datum_zeit_von, :datum_zeit_bis

    #def self.from_dienst dienst
    #  Zeitspanne.new(dienst.datum_zeit_von.to_date, dienst.datum_zeit_von, dienst.datum_zeit_bis, from_dienst: true)
    #end

    def self.relative datum, zeit_von, zeit_bis
      self.new(create_time(datum, zeit_von), create_time(datum, zeit_bis, zeit_von[0] > zeit_bis[0]))
    end

    ##
    # Hilfsfunktion um ein Timeobjekt zu erzeugen
    def self.create_time(datum, zeit, next_day = false)
      datum = datum +1 if next_day
      Time.new(datum.year, datum.month, datum.day, zeit[0], zeit[1])
    end


    def initialize(zeit_von, zeit_bis)
      raise(ArbeitszeitkontoError, "zeit_von ist grösser als zeit_bis") if zeit_von > zeit_bis
      raise(ArbeitszeitkontoError, "zeit_von und zeit_bis sind gleich") if zeit_von == zeit_bis
      @datum_zeit_von = zeit_von
      @datum_zeit_bis = zeit_bis
    end

    def datum
      datum_zeit_von.to_date
    end

    ##
    # liefert die Überlappung der beiden Zeitspannen in Sekunden
    #
    #
    # Fall 1: komplette Überlappung
    # ========================
    # kandidat :   |-----|
    # self     : |---------|

    # Fall 2: Teilweise Überlappung
    # ========================
    # kandidat : |-----|
    # self     :    |---------|
    #------------------------------
    # kandidat :        |-----|
    # self     : |---------|
    #
    #------------------------------
    # kandidat : |----------|
    # self     :    |-----|
    #
    # Fall 3 Keine überlappung
    # ========================
    # kandidat : |-----|
    # self     :        |---------|
    #------------------------------
    # kandidat :          |-----|
    # self     : |--------|
    #

    ##
    def overlap(kandidat)
      seconds = 0
      reste = []
      case
        # Fall 1: komplette Überlappung
        when kandidat.datum_zeit_von >= datum_zeit_von && kandidat.datum_zeit_bis <= datum_zeit_bis
          #puts "#{self.class.name}#overlap: komplette Überlappung"
          seconds = kandidat.datum_zeit_bis - kandidat.datum_zeit_von
          # rest3: keine
        # Fall 2a: Teilüberlappung
        when kandidat.datum_zeit_von < datum_zeit_von && kandidat.datum_zeit_bis > datum_zeit_von && kandidat.datum_zeit_bis <= datum_zeit_bis
          #puts "#{self.class.name}#overlap: Teilweise Überlappung: vorne"
          seconds = kandidat.datum_zeit_bis - datum_zeit_von
          reste << Zeitspanne.new(kandidat.datum_zeit_von, datum_zeit_von)
          # rest: kandidat.datum_zeit_von - self.datzm_zeit_von
        # Fall 2b: Teilüberlappung
        when kandidat.datum_zeit_von > datum_zeit_von && kandidat.datum_zeit_von < datum_zeit_bis
          #puts "#{self.class.name}#overlap: Teilweise Überlappung: hinten"
          seconds = datum_zeit_bis - kandidat.datum_zeit_von
          reste << Zeitspanne.new(datum_zeit_bis, kandidat.datum_zeit_bis)
        # Fall 2c: Teilüberlappung: Kandidat lappt über beide Ende
        when kandidat.datum_zeit_von < datum_zeit_von && kandidat.datum_zeit_bis > datum_zeit_bis
          #puts "#{self.class.name}#overlap: Fall 2c: Teilüberlappung: Kandidat lappt über beide Ende  "
          seconds = datum_zeit_bis - datum_zeit_von
          reste << Zeitspanne.new(kandidat.datum_zeit_von, datum_zeit_von)
          reste << Zeitspanne.new(datum_zeit_bis, kandidat.datum_zeit_bis)
        # Fall 3: Keine Überlappung
        when kandidat.datum_zeit_bis <= datum_zeit_von || kandidat.datum_zeit_von >= datum_zeit_bis
          #puts "#{self.class.name}#overlap: Keine Überlappung"
          second = 0
          reste << kandidat
        else
          raise ArbeitszeitkontoError, "overlap: Unbekannte Situation"
      end
      [seconds, reste]
    end

    def to_s
      "#{d_to_s(datum_zeit_von)} - #{d_to_s(datum_zeit_bis)}"
    end

    private

    def d_to_s dt
      dt.strftime("%Y-%m-%d %H:%M:%S")
    end

  end
end
end
